From abb22e19d114855de995c86ad749f31561739353 Mon Sep 17 00:00:00 2001
From: Jon Griffiths <jon_p_griffiths@yahoo.com>
Date: Mon, 19 Feb 2024 18:46:47 +1300
Subject: [PATCH] tx: check absolute tx limits only when constructing from
 scratch

Allow as many inputs/outputs as are present if we are building a tx from its
serialized bytes. Since we fully analyze the tx before building it, we know
that it can fit in memory/is not impossible to create for DOS checking
purposes. This allows us to process live txs such as a recent testnet tx
with ~100k outputs.

We retain the limit on constructing new txs from scratch, since wally is
meant only for making txs that fit within standardness rules. This
catches cases where a tx is constructed in parts or allocated up front
based on an externally provided number of inputs/outputs.

Update the comments to reflect that the limits are for standardness and
not block size.
---
 src/psbt.c        | 2 +-
 src/transaction.c | 6 +++---
 2 files changed, 4 insertions(+), 4 deletions(-)

diff --git a/src/psbt.c b/src/psbt.c
index bb6b3349233042a9ce83b5dccccd46b5536a0290..4acde0efce6be9f8d5035209b774424fd2c6c4ee 100644
--- a/src/psbt.c
+++ b/src/psbt.c
@@ -1154,7 +1154,7 @@ static int psbt_init(uint32_t version, size_t num_inputs, size_t num_outputs,
     if ((version != PSBT_0 && version != PSBT_2) || !psbt_out)
         return WALLY_EINVAL; /* Only v0/v2 are specified/supported */
     if (num_inputs > TX_MAX_INPUTS || num_outputs > TX_MAX_OUTPUTS)
-        return WALLY_EINVAL; /* Resulting tx could not fit in a block */
+        return WALLY_EINVAL; /* Resulting tx would be non-standard */
 #ifdef BUILD_ELEMENTS
     if (flags & ~WALLY_PSBT_INIT_PSET ||
         (flags & WALLY_PSBT_INIT_PSET && version != PSBT_2))
diff --git a/src/transaction.c b/src/transaction.c
index 23e81cdfd929e27757fc35f92fcb42dff7c1a2d2..66cff32cfa786eff6a06101c7149945eed4e0d2d 100644
--- a/src/transaction.c
+++ b/src/transaction.c
@@ -1162,9 +1162,6 @@ static int tx_init_alloc(uint32_t version, uint32_t locktime,
     struct wally_tx_output *new_outputs = NULL;
 
     OUTPUT_CHECK;
-    if (inputs_allocation_len > TX_MAX_INPUTS ||
-        outputs_allocation_len > TX_MAX_OUTPUTS)
-        return WALLY_EINVAL; /* Tx cannot fit in a block: invalid */
     OUTPUT_ALLOC(struct wally_tx);
 
     if (inputs_allocation_len) {
@@ -1202,6 +1199,9 @@ int wally_tx_init_alloc(uint32_t version, uint32_t locktime,
                         size_t outputs_allocation_len,
                         struct wally_tx **output)
 {
+    if (inputs_allocation_len > TX_MAX_INPUTS ||
+        outputs_allocation_len > TX_MAX_OUTPUTS)
+        return WALLY_EINVAL; /* Non-standard tx: invalid */
     return tx_init_alloc(version, locktime,
                          inputs_allocation_len, outputs_allocation_len,
                          TX_MAX_INPUTS_ALLOC, TX_MAX_OUTPUTS_ALLOC, output);
